/*
 * Counter.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Please do not redistribute without obtaining permission.
 */

package nodenet;

/**
 * An interface for NodeBehaviors that keep track of the number of
 * objects they have seen.  By implementing this interface,
 * <code>SimulationPanel</code> will display the value returned by
 * <code>getCount()</code> whenever the node is displayed. This class
 * replaces CountingNodeBehavior<p>
 *
 * @author Patrick G. Heck gus.heck@olin.edu
 * @version $Id: Counter.java,v 1.3 2004/01/14 21:43:17 gus Exp $
 */
public interface Counter {
    
    /**
     * Returns the number of packets generated, moved or recieved as
     * appropriate.
     *
     * @return The number of packets counted sofar.
     */
    public int getCount();
    
    /**
     * Resets the count.
     */
    public void resetCount();
}

/*
 * $Log: Counter.java,v $
 * Revision 1.3  2004/01/14 21:43:17  gus
 * more javadoc, plus reformat
 *
 * Revision 1.2  2004/01/14 20:23:21  gus
 * Javadoc and comment cleanup
 *
 * Revision 1.1  2004/01/13 19:35:27  gus
 * Simulation Panel, channel and node are all Resettable now.
 * CountingNodeBehavior refactored into Counter
 *
 */




