/*
 * DTDResolver.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Created on January 8, 2004, 5:24 PM
 */

package nodenet;

import org.xml.sax.InputSource;
import org.xml.sax.EntityResolver;

import java.io.InputStreamReader;

/**
 * This class allows us to validate against a DTD that is found on
 * the classpath. This is important because otherwise we would need to
 * deploy the DTD on our website and parsers would look it up there, or the
 * DTD woud have to be placed in a specific location in the filesystem where
 * we could find it. In the first case the program would break on computers
 * not currently connected to the internet, and in the latter case would
 * impose an unacceptable configuration burden on students and the professors
 * who guide them.
 *
 *<p>The Apache Crimson parser on which the default jvm's rely seems to have
 * some quirks. One is that the value of sys must be a valid URI (as per
 * the java.net.URI class with a known scheme) Since this is entirely
 * irrelevant for the classloader, the KLUDGE constant must be prepended to
 * the sys string, otherwise crimson will throw exceptions before invoking
 * this EntityResolver. This appears to violate the spec in the javadoc but
 * I can't do anything about it. If Java switches to Xerces2 this may
 * not be neccesary.
 *
 * @author  Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: DTDResolver.java,v 1.4 2004/01/15 18:38:38 gus Exp $
 */
public class DTDResolver implements org.xml.sax.EntityResolver {
    
    /**
     * A dummy string to prepend to the system DTD entity to
     * workaround a bug in the crimson parser. It is also used to workaround
     * a problem with InputSource which refuses to use any system resource
     * unless systemId is set to a valid URI.
     */
    public static final String KLUDGE = "file:///ignoreThisKludge/";
    
    /** Creates a new instance of DTDResolver */
    public DTDResolver() {
    }
    public org.xml.sax.InputSource resolveEntity(String pub, String sys) {
        // this undoes the name resolution kludge see
        // SimulationPanel.templateXML for details
        sys = sys.substring(KLUDGE.length());
        InputSource is = null;
        try {
            InputStreamReader isr = new InputStreamReader(
            SimulationPanel.class.getResourceAsStream(sys),"UTF8");
            is = new InputSource(isr);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return is;  // if null default resolution is used
    }
}


