/*
 * IntermediateNodeBehavior.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Please do not redistribute without obtaining permission.
 */

package nodenet;

import java.io.*;
import java.util.Vector;

/**
 * This is an improved version of the DefaultNode.
 * It does not drop packets and chooses judiciously among
 * available inputs and outputs.<p>
 *
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: IntermediateNodeBehavior.java,v 1.8 2004/01/14 20:23:21 gus Exp $
 */
class IntermediateNodeBehavior implements NodeBehavior, Resettable{
    
    public static final String BEHAVIOR_NAME = "Intermediate";
    
    
    private boolean readyToWrite;
    private int nextInIndex, nextOutIndex;
    private Object packetToBeSent;
    
    private void readFromChannel(InputChannelVector inputChannels) {
        
        // If there's no input stream, exit right away.
        if (inputChannels.size() == 0) return;
        for (int i = 0; i < inputChannels.size() && !readyToWrite; i++) {
            try {
                packetToBeSent =
                inputChannels.elementAt(nextInIndex).readObject(); // ()?
                readyToWrite = true;
            }
            catch(ChannelEmptyException exc) {
                //System.out.println("Tried to read from EmptyChannel!");
            } catch(ChannelDisabledException exc2) {
                System.out.println("Read from Disabled Channel!");
            } catch(ArrayIndexOutOfBoundsException exc3) {
                System.out.println("Shoot the Programmer!\n" + exc3 );
            }
            
            nextInIndex = (nextInIndex + 1) % inputChannels.size();
            // or, if you want to be really grotesque (Todd says cool):
            // nextInIndex = ++nextInputStream % inputChannels.size();
        }
    }
    
    
    private void writeToChannel(OutputChannelVector outputChannels) {
        
        if (outputChannels.size() == 0) return;
        for (int i = 0; i < outputChannels.size() && readyToWrite; i++) {
            try {
                outputChannels.elementAt(nextOutIndex).writeObject( packetToBeSent ); // ()?
                readyToWrite = false;
            } catch(ChannelFullException exc) {
                System.out.println("Full Channel!");
            } catch(ChannelDisabledException exc2) {
                System.out.println("Wrote to Disabled Channel!");
            } catch(ArrayIndexOutOfBoundsException exc3) {
                System.out.println("Shoot the Programmer!\n" + exc3);
            }
            
            nextOutIndex = ++nextOutIndex % outputChannels.size();
            // or, if you want to be verbose and clear (Lynn says cool):
            // nextOutIndex = (nextOutIndex + 1) % outputChannels.size();
        }
    }
    
    public void transmitPacket(InputChannelVector inputs, OutputChannelVector outputs) {
        if (readyToWrite) {
            writeToChannel( outputs );
        } else  { /* not readyToWrite */
            readFromChannel( inputs );
        }
    }
    
    public void reset() {
        this.packetToBeSent = null;
        this.nextInIndex = 0;
        this.nextOutIndex = 0;
        this.readyToWrite = false;
    }
    
}

/*
 * $Log: IntermediateNodeBehavior.java,v $
 * Revision 1.8  2004/01/14 20:23:21  gus
 * Javadoc and comment cleanup
 *
 * Revision 1.7  2004/01/13 19:35:27  gus
 * Simulation Panel, channel and node are all Resettable now.
 * CountingNodeBehavior refactored into Counter
 *
 * Revision 1.6  2004/01/09 20:00:01  gus
 * Empty channel messages are annoying can them
 *
 * Revision 1.5  2004/01/07 19:49:43  gus
 * Cleanup Exception Handling
 *
 * Revision 1.4  2004/01/07 18:32:47  gus
 * Ripped out ControlPanel and NodeNetRadioButton.
 * Replaced them with NodeTypeSelector, which provides
 * The loaded User NodeBehavior types as an indexed
 * property. Simulation panel now implements PropertyChangeListener
 * to listen for changes in the selected node behavior. The
 * buttons in the display are now JToggleButtons, and will
 * use the public field BEHAVIOR_NAME to lable the buttons
 * if it is declared for any NodeBehavior.
 *
 * Revision 1.3  2003/02/14 22:21:39  gus
 * spacing
 *
 * Revision 1.2  2003/02/14 22:20:58  gus
 * I hate exceptions that are simply ignored.
 *
 * Revision 1.1  2002/06/13 17:33:21  gus
 * Moved all java files into the nodenet directory (who let them out anyway?)
 *
 * Revision 1.1.1.1  2002/06/05 21:56:35  root
 * CS101 comes to Olin finally.
 *
 * Revision 1.4  2000/05/09 06:03:54  mharder
 * Changed packagename from nodeNet to nodenet.
 *
 * Revision 1.3  1999/08/04 09:08:52  jsmthng
 * Added javadoc comments to InputChannelVector and OutputChannelVector;
 * finished updating the rest of the nodeNet package to reflect new
 * changes in name and code.
 *
 * Modified index.html to reflect the new nodeNet code, as well as to
 * clarify some parts of the problem set.
 *
 * Revision 1.1  1999/07/30 01:09:21  jsmthng
 * Renaming BinSort package to nodeNet; moving directories and files as
 * necessary.
 *
 * Revision 1.5  1998/08/12 19:29:36  tparnell
 * Another pass after comments from las & natashao.  Added support to
 * dynamically add NodeBehaviors.  Add keyboard shortcuts to menus.  Added
 * workaround to jdk bug relating to lighweight components.  Misc other
 * bugfixes.
 *
 * Revision 1.3  1998/08/10 17:45:50  tparnell
 * Revision to use JDK1.2 and Swing.  Redesign of GUI.  Removed old kludge
 * for file I/O and replaced with object serialization.  Channel no longer
 * requires animacy.  Removed unnessary dependencies between classes.
 * Added ability to configure channel's latency and capacity.  Added
 * javadoc to all files.  General code cleanup.
 *
 */
