/*
 * Main.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Please do not redistribute without obtaining permission.
 */

package nodenet;

//import com.sun.java.swing.*;
import javax.swing.*;
import java.applet.*;
import java.awt.event.*;
import java.awt.*;

import java.util.List;
import java.util.ArrayList;

import nodenet.registrar.UniqueNodeNamePolicy;
import nodenet.registrar.NodeNameRegistry;
import nodenet.registrar.NodeNameRegistrar;

/**
 * Main defines the <code>public static void main(String[])</code>
 * method.  It can also be run as an applet.<p>
 *
 * Copyright (c) 1998 Massachusetts Institute of Technology
 *
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: Main.java,v 1.9 2004/01/14 21:43:17 gus Exp $
 */
public class Main extends JApplet {
    
    private NodeNetFrame me;
    
    public static List loadedBehaviors = new ArrayList();
    /**
     * Starts a new simulation.  All arguments are treated as
     * NodeBehaviors and will be available to the user.
     */
    public static void main(String[] args) {
        
        // process the args and load any nodeNameRegistar classes we
        // find. If we find more than one we will get an illegal state
        // exception which should not be caught.
        
        for (int i=0; i<args.length;i++){
            try {
                NodeNameRegistry.getInstance().
                setPolicy((NodeNameRegistrar)Class.forName(args[i]).newInstance());
            } catch (ClassCastException cce) {
                //This case is for classes that were found but were not
                // of type NodeNameRegistrar. Save them since they may
                // be behavior classes.
                loadedBehaviors.add(args[i]);
            } catch (ClassNotFoundException cnfe) {
                System.out.println("Specified class name not found:"+args[i]);
            } catch (InstantiationException ie) {
                System.out.println("Failed to instantiate:"+args[i]);
            } catch (IllegalAccessException iae) {
                iae.printStackTrace();
                System.exit(1);
            }
        }
        
        // if we didn't find a registrar on the command line use the
        // default UniqueNodeNamePolicy
        if (!NodeNameRegistry.getInstance().isPolicySet()) {
            System.out.println("Defaulting to UniqueNodeNamePolicy to " +
            "control node naming");
            
            NodeNameRegistry.getInstance().setPolicy(new UniqueNodeNamePolicy());
        }
        
        // Now put the behaviors back in an array so we can pass it to
        // NodeNetFrame
        if (loadedBehaviors.size() > 0) {
            args = new String[loadedBehaviors.size()];
            Object[] holder = loadedBehaviors.toArray();
            for(int i=0; i<loadedBehaviors.size();i++) {
                args[i] = (String) holder[i];
            }
        } else {
            args = new String[] {};
        }
        
        // set the look and feel...
        //( preexisting code: I might can this later for simplicity sake)
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException cnfe) {
            // stick with the default if we can't find it.
        } catch (InstantiationException ie) {
            // stick witht he defualt if it seems to be broken
        } catch (IllegalAccessException iae) {
            // stick with the default if we arn't allowed to load it.
        } catch (UnsupportedLookAndFeelException ulafe) {
            // stick with the default if the jvm won't support the system LaF
        }
        
        // and finally build our frame passing in the remaining
        // command line arguments.
        NodeNetFrame nnf = new NodeNetFrame(args);
    }
    
    /**
     * Initilization as an applet.
     */
    public void init() {
        NodeNameRegistry.getInstance().setPolicy(new UniqueNodeNamePolicy());
        Button b = new Button("Show/Hide");
        String[] foo = new String[1];
        foo[0] = "nodenet.IntermediateNodeBehavior";
        this.me = new NodeNetFrame(foo);
        b.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (Main.this.me.isVisible())
                    Main.this.me.setVisible(false);
                else Main.this.me.show();
            }
        });
        this.getContentPane().add(b);
    }
}

/*
 * $Log: Main.java,v $
 * Revision 1.9  2004/01/14 21:43:17  gus
 * more javadoc, plus reformat
 *
 * Revision 1.8  2004/01/14 20:23:21  gus
 * Javadoc and comment cleanup
 *
 * Revision 1.7  2004/01/07 18:32:47  gus
 * Ripped out ControlPanel and NodeNetRadioButton.
 * Replaced them with NodeTypeSelector, which provides
 * The loaded User NodeBehavior types as an indexed
 * property. Simulation panel now implements PropertyChangeListener
 * to listen for changes in the selected node behavior. The
 * buttons in the display are now JToggleButtons, and will
 * use the public field BEHAVIOR_NAME to lable the buttons
 * if it is declared for any NodeBehavior.
 *
 * Revision 1.6  2004/01/06 18:44:25  gus
 * Make the names of the loaded behaviors available
 *
 * Revision 1.5  2003/02/25 15:23:34  gus
 * fix class cast exception with casting of arrays.
 * Add comments to illucidate the main method.
 *
 * Revision 1.4  2003/02/21 18:07:49  gus
 * Search the commandline for an implementation of NodeNameRegistrar, and use it
 * implement our naming policy for NodeNameRegistry, then pass any arguments
 * that were loadable classes but not NodeNameRegistrars to the constructor of
 * NodeNetFrame.
 *
 * Also, cleaned up the horribly ugly catch(Exception e)
 *
 * Revision 1.3  2002/08/20 15:39:18  gus
 * corrected package name
 *
 * Revision 1.2  2002/06/13 19:32:43  gus
 * Commented out com.sun.java.swing package references, and updated obsolete
 * imorts and references into cs101 package.
 *
 * Revision 1.1  2002/06/13 17:33:21  gus
 * Moved all java files into the nodenet directory (who let them out anyway?)
 *
 * Revision 1.1.1.1  2002/06/05 21:56:35  root
 * CS101 comes to Olin finally.
 *
 * Revision 1.4  2000/05/09 06:03:54  mharder
 * Changed packagename from nodeNet to nodenet.
 *
 * Revision 1.3  1999/08/04 09:08:52  jsmthng
 * Added javadoc comments to InputChannelVector and OutputChannelVector;
 * finished updating the rest of the nodeNet package to reflect new
 * changes in name and code.
 *
 * Modified index.html to reflect the new nodeNet code, as well as to
 * clarify some parts of the problem set.
 *
 * Revision 1.1  1999/07/30 01:09:22  jsmthng
 * Renaming BinSort package to nodeNet; moving directories and files as
 * necessary.
 *
 * Revision 1.6  1999/01/21 20:57:06  tparnell
 * it worked
 *
 * Revision 1.5  1998/08/14 16:17:43  tparnell
 * now appearing in applet version
 *
 * Revision 1.4  1998/08/12 19:29:36  tparnell
 * Another pass after comments from las & natashao.  Added support to
 * dynamically add NodeBehaviors.  Add keyboard shortcuts to menus.  Added
 * workaround to jdk bug relating to lighweight components.  Misc other
 * bugfixes.
 *
 * Revision 1.3  1998/08/10 17:45:50  tparnell
 * Revision to use JDK1.2 and Swing.  Redesign of GUI.  Removed old kludge
 * for file I/O and replaced with object serialization.  Channel no longer
 * requires animacy.  Removed unnessary dependencies between classes.
 * Added ability to configure channel's latency and capacity.  Added
 * javadoc to all files.  General code cleanup.
 *
 */
