/*
 * BehaviorProvider.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Created on January 8, 2004, 11:34 AM
 */

package nodenet;

import java.beans.PropertyChangeListener;

/**
 * This interface indicates that a class can provide a list of all loaded
 * behaviors, and notify listeners of changes in the selected behavior.
 *
 * @author  Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: NodeBehaviorProvider.java,v 1.4 2004/01/14 21:43:17 gus Exp $
 */
public interface NodeBehaviorProvider {
    
    /**
     * Provide a list of <strong>all user loaded behaviors in the
     * entire application</strong>.
     *
     * @return Every {@link NodeBehavior} class currently loaded
     *         for use in node creation
     */
    public Class[] getNodeBehaviors();
    
    /**
     * Regeister a class that wants to know when the selected node behavior
     * type is changed. As soon as the client is registered an event is
     * generated indicating a change from null to the current value to bring
     * the listener up to date.
     *
     * @param l  The object that wishes to listen for events
     */
    public void addPropertyChangeListener(PropertyChangeListener l);
    
    /**
     * Unregeister a class that no longer wants to know when the selected
     * node behavior type is changed.
     *
     * @param l  The object that is no longer interested in events
     */
    public void removePropertyChangeListener(PropertyChangeListener l) ;
    
}
