/*
 * NodeNetIcon.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Created on January 6, 2004, 3:04 PM
 * Please do not redistribute without obtaining permission.
 */

package nodenet;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Component;

import javax.swing.Icon;

/**
 * An icon to draw nodes reflecting their selection status. This was formerly
 * an inner class of ControlPanel.
 *
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @author  Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: NodeNetIcon.java,v 1.3 2004/01/14 21:43:17 gus Exp $
 */
public class NodeNetIcon implements Icon {
    
    private Color color;
    private boolean state;
    
    /** Creates a new instance of NodeNetIcon */
    public NodeNetIcon() {
    }
    
    /**
     * Creates a new instance of NodeNetIcon with the specified
     * color and state.
     */
    public NodeNetIcon(Color c, boolean s) {
        this.color = c;
        this.state = s;
    }
    
    // implementing Icon
    public void paintIcon(Component c, Graphics g, int x, int y) {
        int width = getIconWidth();
        int height = getIconHeight();
        g.setColor(color);
        g.fillOval(x, y, width, height);
        if (this.state) {
            g.setColor(Color.black);
            g.drawRect(x, y, width, height);
        }
    }
    
    // implementing Icon
    public int getIconWidth() {
        return Node.GUISIZE;
    }
    
    // implementing Icon
    public int getIconHeight() {
        return Node.GUISIZE;
    }
    
    /**
     * Getter for property color.
     *
     * @return Value of property color.
     *
     */
    public java.awt.Color getColor() {
        return color;
    }
    
    /**
     * Setter for property color.
     *
     * @param color New value of property color.
     *
     */
    public void setColor(java.awt.Color color) {
        this.color = color;
    }
    
}
