/*
 * NodeTypeSelector.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Created on January 6, 2004, 12:18 PM
 * Please do not redistribute without obtaining permission.
 */

package nodenet;

import java.awt.Color;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.EventQueue;

import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.ActionListener;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;

import java.util.Iterator;
import java.util.ArrayList;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import javax.swing.JPanel;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.JPopupMenu;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;

/**
 * Implements a custom component that manages the loaded NodeBehavior classes
 * and allows the user to select one behavior. The selected behavior is
 * a simple bound property, and the list of loaded NodeBehavior
 * classes is an indexed property of this class.
 *
 * @author  Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: NodeTypeSelector.java,v 1.12 2004/01/16 20:48:59 gus Exp $
 */
public class NodeTypeSelector extends JPanel implements NodeBehaviorProvider {
    
    /**
     * indecates that the array of nodeBehaviors should be re-sized by +1 and
     * the new behavior added in the next position.
     */
    public static int NEXT = -99;  // using -1 might hide loop errors!
    
    /** Utility field used by bound properties. */
    private java.beans.PropertyChangeSupport propertyChangeSupport =  new PropertyChangeSupport(this);
    
    /** Holds value of property nodeBehaviors. */
    private Class[] nodeBehaviors = new Class[] {};
    
    /** Holds value of property selectedBehavior. */
    private java.lang.Class selectedBehavior;
    
    /** Holds the default generator behavior */
    private java.lang.Class generator = GeneratorNodeBehavior.class;
    
    /** Holds the default terminator behavior */
    private java.lang.Class terminator = TerminatorNodeBehavior.class;
    
    /** holds the minimum size to display ourselves*/
    private java.awt.Dimension minSize;
    
    /** Creates new form NodeTypeSelector */
    public NodeTypeSelector() {
        initComponents();
        initBehaviorButtons();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        behaviorButtons = new javax.swing.ButtonGroup();
        popup = new javax.swing.JPopupMenu();
        moveUpMenuItem = new javax.swing.JMenuItem();
        moveDownMenuItem = new javax.swing.JMenuItem();
        buttonScrollPane = new javax.swing.JScrollPane();
        buttonPanel = new javax.swing.JPanel();

        moveUpMenuItem.setText("Move Up");
        moveUpMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moveUpMenuItemActionPerformed(evt);
            }
        });

        popup.add(moveUpMenuItem);

        moveDownMenuItem.setText("Move Down");
        moveDownMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moveDownMenuItemActionPerformed(evt);
            }
        });

        popup.add(moveDownMenuItem);

        setLayout(new java.awt.GridLayout(1, 0));

        buttonScrollPane.setAlignmentX(0.0F);
        buttonPanel.setLayout(new javax.swing.BoxLayout(buttonPanel, javax.swing.BoxLayout.Y_AXIS));

        buttonScrollPane.setViewportView(buttonPanel);

        add(buttonScrollPane);

    }//GEN-END:initComponents
    
    private void moveDownMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moveDownMenuItemActionPerformed
        // Add your handling code here:
        Object button = ((JMenuItem)evt.getSource()).getClientProperty("button");
        if (button instanceof JToggleButton) {
            Class behavior = (Class)((JToggleButton)button).getClientProperty("myBehavior");
            moveDown(behavior);
            refreshButtons();
        }
    }//GEN-LAST:event_moveDownMenuItemActionPerformed
    
    private void moveUpMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moveUpMenuItemActionPerformed
        // Add your handling code here:
        Object button = ((JMenuItem)evt.getSource()).getClientProperty("button");
        if (button instanceof JToggleButton) {
            Class behavior = (Class)((JToggleButton)button).getClientProperty("myBehavior");
            moveUp(behavior);
            refreshButtons();
        }
    }//GEN-LAST:event_moveUpMenuItemActionPerformed
    
    
    /**
     * Determine if a class is a behavior loaded by the user
     */
    
    private boolean isUserBehavior(Class behavior) {
        for (int i = 0; i < this.nodeBehaviors.length; i++) {
            if (this.nodeBehaviors[i].equals(behavior)) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Determine if the specified class is last in the list of node behaviors.
     */
    private boolean isLast(Class behavior) {
        if (this.nodeBehaviors.length > 0) {
            return this.nodeBehaviors[this.nodeBehaviors.length-1].equals(behavior);
        } else {
            return false;
        }
    }
    
    /**
     * Determine if the specified class is first in the list of node behaviors.
     */
    private boolean isFirst(Class behavior) {
        if (this.nodeBehaviors.length > 0) {
            return this.nodeBehaviors[0].equals(behavior);
        } else {
            return false;
        }
    }
    
    /**
     * Move the behavior Down one position in the list if possible.
     */
    
    private void moveDown(Class behavior) {
        if (isLast(behavior) || (this.nodeBehaviors.length == 1)) {
            return;
        } else {
            for (int i = 0; i < this.nodeBehaviors.length; i++) {
                if (this.nodeBehaviors[i].equals(behavior)) {
                    this.nodeBehaviors[i] = this.nodeBehaviors[i + 1];
                    this.nodeBehaviors[i + 1] = behavior;
                    break;
                }
            }
        }
    }
    
    /**
     * Move the behavior up one position in the list if possible.
     */
    private void moveUp(Class behavior) {
        if (isFirst(behavior) || (this.nodeBehaviors.length == 1)) {
            return;
        } else {
            for (int i = 0; i < this.nodeBehaviors.length; i++) {
                if (this.nodeBehaviors[i].equals(behavior)) {
                    this.nodeBehaviors[i] = this.nodeBehaviors[i - 1];
                    this.nodeBehaviors[i - 1] = behavior;
                    break;
                }
            }
        }
    }
    
    /** Adds a PropertyChangeListener to the listener list.
     * @param l The listener to add.
     *
     */
    public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
        propertyChangeSupport.addPropertyChangeListener(l);
        PropertyChangeEvent pce;
        pce = new PropertyChangeEvent(this,"selectedBehavior",
        null, selectedBehavior);
        l.propertyChange(pce);
    }
    
    /** Removes a PropertyChangeListener from the listener list.
     * @param l The listener to remove.
     *
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        propertyChangeSupport.removePropertyChangeListener(l);
    }
    
    /** Indexed getter for property nodeBehaviors.
     * @param index Index of the property.
     * @return Value of the property at <CODE>index</CODE>.
     *
     */
    public Class getNodeBehaviors(int index) {
        return this.nodeBehaviors[index];
    }
    
    /** Getter for property nodeBehaviors.
     * @return Value of property nodeBehaviors.
     *
     */
    public Class[] getNodeBehaviors() {
        return this.nodeBehaviors;
    }
    
    /**
     * Indexed setter for property nodeBehaviors. Passing an index of
     * {@link NodeTypeSelector#NEXT} will automatically increase the
     * array backing this property by one, and add the specified class at
     * the end of the array. Note that this is backed by an array, because
     * that keeps this a legal java bean. The alternative would be to back it
     * with a List and convert arrays to lists in
     * {@link #setNodeBehaviors(Class[])}.
     *
     * @param index Index of the property.
     * @param nodeBehaviors New value of the property at <CODE>index</CODE>.
     *
     */
    public void setNodeBehaviors(int index, Class nodeBehaviors) {
        setNodeBehaviors(index, nodeBehaviors, true);
    }
    
    private void setNodeBehaviors(int index, Class nodeBehaviors,
    boolean refresh) {
        
        checkClass(nodeBehaviors);
        int realIndex = index;
        if (index == NEXT) {
            Class[] temp = new Class[this.nodeBehaviors.length + 1];
            System.arraycopy(this.nodeBehaviors, 0, temp, 0,
            this.nodeBehaviors.length);
            realIndex = temp.length-1;
            this.nodeBehaviors = temp;
        }
        this.nodeBehaviors[realIndex] = nodeBehaviors;
        if (refresh) {
            refreshButtons();
        }
    }
    /** Setter for property nodeBehaviors.
     * @param nodeBehaviors New value of property nodeBehaviors.
     *
     */
    public void setNodeBehaviors(Class[] nodeBehaviors) {
        this.nodeBehaviors = nodeBehaviors;
    }
    
    /** Getter for property selectedBehavior.
     * @return Value of property selectedBehavior.
     *
     */
    public java.lang.Class getSelectedBehavior() {
        return this.selectedBehavior;
    }
    
    /** Setter for property selectedBehavior.
     * @param selectedBehavior New value of property selectedBehavior.
     *
     */
    public synchronized void setSelectedBehavior(Class selectedBehavior) {
        java.lang.Class oldSelectedBehavior = this.selectedBehavior;
        this.selectedBehavior = selectedBehavior;
        propertyChangeSupport.firePropertyChange("selectedBehavior",
        oldSelectedBehavior, selectedBehavior);
    }
    
    /**
     * Create a button to represent a behavior.
     */
    private JToggleButton createButton(Class behavior) {
        JToggleButton btn = null;
        try {
            Field[] fields = behavior.getFields();
            for (int i = 0; i < fields.length; i++) {
                if ("BEHAVIOR_NAME".equals(fields[i].getName())) {
                    btn = new JToggleButton((String)fields[i].get(behavior));
                }
            }
        } catch (SecurityException se) {
            // just ignore...
        } catch (IllegalAccessException iae) {
            // just ignore...
        }
        
        if (btn == null) {
            btn = new JToggleButton(behavior.getName());
        }
        btn.putClientProperty("myBehavior", behavior);
        Color color = Node.getColorFromBehavior(behavior.getName());
        btn.setIcon(new NodeNetIcon(color, false));
        btn.setSelectedIcon(new NodeNetIcon(color, true));
        btn.setHorizontalAlignment(SwingConstants.LEFT);
        btn.setMaximumSize(new Dimension(1024,30));
        
        btn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                setSelectedBehavior((Class)
                ((JToggleButton)ae.getSource()).
                getClientProperty("myBehavior"));
            }
        });
        btn.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                maybeShowPopup(e);
            }
            
            public void mouseReleased(MouseEvent e) {
                maybeShowPopup(e);
            }
            
            private void maybeShowPopup(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    Class b = (Class)((JToggleButton)e.getSource()).
                    getClientProperty("myBehavior");
                    if (isFirst(b) || !isUserBehavior(b)) {
                        NodeTypeSelector.this.moveUpMenuItem.setEnabled(false);
                    } else {
                        NodeTypeSelector.this.moveUpMenuItem.setEnabled(true);
                    }
                    if (isLast(b) || !isUserBehavior(b)) {
                        NodeTypeSelector.this.
                        moveDownMenuItem.setEnabled(false);
                    } else {
                        NodeTypeSelector.this.
                        moveDownMenuItem.setEnabled(true);
                    }
                    NodeTypeSelector.this.moveDownMenuItem.
                    putClientProperty("button", e.getSource());
                    NodeTypeSelector.this.moveUpMenuItem.
                    putClientProperty("button", e.getSource());
                    NodeTypeSelector.this.popup.show(e.getComponent(),
                    e.getX(), e.getY());
                }
            }
        });
        
        return btn;
    }
    
    /**
     * checks a class to see if it is a valid and working node behavior.
     */
    private void checkClass(Class behavior) {
        String behaviorName = behavior.getName();
        
        //Check to see if it implements NodeBehavior
        Class nb = NodeBehavior.class;
        Class[] interfaces = behavior.getInterfaces();
        boolean found = false;
        for (int i=0; i<interfaces.length; i++) {
            if (interfaces[i].equals(nb)) {
                found = true;
                break;
            }
        }
        if (!found) {
            throw new IllegalArgumentException(behaviorName +
            " does not implement NodeBehavior.");
        }
        
        // Check that it isn't abstract or an interface
        int modifiers = behavior.getModifiers();
        if (Modifier.isAbstract(modifiers)
        || Modifier.isInterface(modifiers)) {
            throw new IllegalArgumentException(behaviorName +
            " must be a non-abstract class.");
        }
        
        // Check that it can be instantiated
        
        try {
            Object o = behavior.newInstance();
        } catch (IllegalAccessException iae) {
            iae.printStackTrace();
            throw new IllegalArgumentException(behaviorName +
            " must have a zero-argument constructor.");
        } catch (InstantiationException ie) {
            // shouldn't happen (we know it's a non-abstract class)
            throw new Error("Error in checkClass algorithm");
        }
        
    }
    
    /**
     * Creates and adds a fresh set of buttons based on the contents of
     * nodeBehaviors. This should only be called from refreshButtons.
     */
    private void addButtons() {
        JToggleButton btn = createButton(generator);
        this.buttonPanel.add(btn);
        this.behaviorButtons.add(btn);
        btn.doClick();
        
        JSeparator sep = new JSeparator(SwingConstants.HORIZONTAL);
        sep.setMaximumSize(new Dimension(2000, 5));
        this.buttonPanel.add(sep);
        
        for (int i = 0; i < nodeBehaviors.length; i++) {
            JToggleButton userBtn = createButton(this.getNodeBehaviors(i));
            this.buttonPanel.add(userBtn);
            this.behaviorButtons.add(userBtn);
        }
        
        sep = new JSeparator(SwingConstants.HORIZONTAL);
        sep.setMaximumSize(new Dimension(2000, 5));
        this.buttonPanel.add(sep);
        
        btn = createButton(terminator);
        this.buttonPanel.add(btn);
        this.behaviorButtons.add(btn);
        //this.setSize(btn.getPreferredSize().width, this.getHeight());
    }
    
    /**
     * This should never ever be called except from refreshButtons.
     * Don't use it. Use refreshButtons() instead. You have been warned.
     */
    private void doRefreshInternal() {
        Component[] ourStuff =
        NodeTypeSelector.this.buttonPanel.getComponents();
        
        for (int i = 0; i < ourStuff.length; i++) {
            NodeTypeSelector.this.buttonPanel.remove(ourStuff[i]);
            try {
                NodeTypeSelector.this.behaviorButtons.
                remove((JToggleButton)ourStuff[i]);
            } catch (ClassCastException cce) {
                // ignore
            }
        }
        NodeTypeSelector.this.addButtons();
        NodeTypeSelector.this.invalidate();
        NodeTypeSelector.this.validate();
        
    }
    
    /**
     * Delete and recreate all the buttons based on the contents of
     * nodeBehaviors. This sort of GUI manipulation needs to be done in the
     * Event Dispatcher thread to avoid deadlock and other nasties in the GUI.
     * (This can occur when the user's thread tries to modify the GUI, at the
     * same time as the Event Dispatch thread, and each winds up holding a lock
     * on something the other one needs). It is rare, but not impossible. In
     * this case I am touching as much as 10=20% of the components in the
     * GUI so I'm doing it the safe way.
     */
    private void refreshButtons() {
        if (!EventQueue.isDispatchThread() ) {
            try {
                EventQueue.invokeAndWait( new Runnable() {
                    public void run() {
                        //System.out.println("invoked");
                        doRefreshInternal();
                    }
                });
            } catch (InterruptedException ie) {
                System.err.println("Interrupted while refreshing buttons");
                
            } catch (java.lang.reflect.InvocationTargetException ite) {
                System.err.println("Exception while refreshing buttons:" + ite);
                ite.printStackTrace();
            }
        } else {
            doRefreshInternal();
        }
    }
    
    /**
     * Load a behavior class if possible. Returns null if the class is not
     * not found.
     */
    public Class loadBehavior(String behaviorName, ClassLoader loader)  {
        Class behavior = null;
        try {
            behavior = Class.forName(behaviorName, true, loader);
            System.out.println(behaviorName + " loaded successfully");
        } catch (ClassNotFoundException cnfe) {
            System.err.println(behaviorName +
            " not found, skipping...");
        }
        return behavior;
    }
    
    /**
     * Initializes the buttons with classes specified on the command line.
     */
    private void initBehaviorButtons() {
        
        if( Main.loadedBehaviors.size() == 0 ) {
            this.setNodeBehaviors(NEXT, IntermediateNodeBehavior.class, false);
        } else {
            for (Iterator iter = Main.loadedBehaviors.iterator();
            iter.hasNext();) {
                Class behavior = loadBehavior((String)iter.next(), this.getClass().getClassLoader());
                if (behavior == null) {
                    continue;
                }
                if (iter.hasNext()) {
                    this.setNodeBehaviors(NEXT,behavior,false);
                } else {
                    this.setNodeBehaviors(NEXT,behavior,true);
                }
            }
        }
        refreshButtons();
        
        //for (int i = 0; i< this.
    }
    
    // I wish I knew how to configure where netbeans puts these.
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup behaviorButtons;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JScrollPane buttonScrollPane;
    private javax.swing.JMenuItem moveDownMenuItem;
    private javax.swing.JMenuItem moveUpMenuItem;
    private javax.swing.JPopupMenu popup;
    // End of variables declaration//GEN-END:variables
    
    
}
