/*
 * Packet.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Created on February 24, 2003, 8:20 AM
 * Please do not redistribute without obtaining permission.
 */

package nodenet;

import java.util.Vector;

/**
 * An object that can be sent across the network. Currently all code
 * ignores the fact that packets are more than Objects, but in future versions
 * the destination and the list of visited nodes may become significant.
 *
 * @author Patrick G. Heck, gus.heck@olin.edu
 * $Id: Packet.java,v 1.6 2004/01/14 21:43:17 gus Exp $
 */
public class Packet implements Cloneable {
    
    private String dest;
    private Vector visited = new Vector();
    
    /** Creates a new instance of Packet */
    public Packet(String dest) {
        this.dest = dest;
    }
    
    
    /**
     * Find out where the packet is headed.
     *
     * @return the destination set for this packet if any
     */
    public String getDest() {
        return this.dest;
    }
    
    /**
     * Return a clone of the Packet. The cloned object is guaranteed to
     * be an independant copy of the current object and not share any
     * instances of mutable classes with the parent object. Immutable
     * classes may or may not be shared. Client code should not rely on
     * sharing of immutable classes since this may break in future
     * implementations.
     *
     * @return a clone of the object.
     */
    public Object clone() {
        Packet tmp;
        try {
            tmp = (Packet) super.clone();
        } catch (CloneNotSupportedException cnse) {
            // The language actually guarantees this won't happen,
            // but we must catch it anyway
            throw new Error("Java is broken! Arghhhhhh... run for your life!");
        }
        
        tmp.visited = getVisitList();
        return tmp;
    }
    
    /**
     * Record that a node was visited.
     *
     * @param nodeName the name of the node to be reocorded
     */
    
    public void logVisit(String nodeName) {
        visited.add(nodeName);
    }
    
    /**
     * Get a list of the nodes visited, if they have been recorded.
     * For this to be useful, the NodeBehaviors must invoke
     * {@link #logVisit(String)} when they handle a packet.
     *
     * @return A <em>copy</em> of the list of the nodes visited.
     */
    public final Vector getVisitList() {
        Vector tmp = new Vector();
        for (int i=0; i<visited.size();i++) {
            // NOTE: casting to String below, to ensure that any attempt to
            // store something other than strings in visited (in later
            // versions) is forced to read this comment, and is aware that
            // this method is used in the clone method. This means that
            // the line below will need to clone objects in the vector
            // to avoid having 2 vectors pointing to the same objects,
            // or use immutable classes (pointing to the same
            // instance of an immutable class is not a problem).
            //
            // The current implementation is acceptable because String is
            // an immutable class.
            tmp.add(((String)visited.get(i)));
        }
        return tmp;
    }
}

/*
 * $Log: Packet.java,v $
 * Revision 1.6  2004/01/14 21:43:17  gus
 * more javadoc, plus reformat
 *
 * Revision 1.5  2004/01/14 21:04:16  gus
 * More javadoc fixes
 *
 * Revision 1.4  2004/01/14 20:23:21  gus
 * Javadoc and comment cleanup
 *
 * Revision 1.3  2003/02/24 17:48:50  gus
 * Add the ability for packets to remember where they have been.
 *
 * Revision 1.2  2003/02/24 16:27:47  gus
 * added Log comment
 *
 */