/*
 * Resettable.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Created on January 12, 2004, 4:09 PM
 * Please do not redistribute without obtaining permission.
 */

package nodenet;

/**
 * An interface indicating that the state of the object can be reset to
 * a standard initial state. This typically means getting rid of packets,
 * setting packet counters to zero and similar re-initializations.
 *
 * @author Patrick G. Heck, gus.heck@olin.edu
 */
public interface Resettable {
    
    /**
     * Return all properties and other state to the initial state of the
     * object so long as such changes do not alter the network structure.
     * For example channels should drop all contained packets, but should
     * not forget what nodes they are attached to. NodeBehaviors should
     * implement this if they hold onto objects read from the channel
     * after transmitPacket is called, otherwise resetting the network
     * will leave some packets held in the nodes.
     *
     */
    public void reset();
    
}
