/*
 * TerminatorNodeBehavior.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Please do not redistribute without obtaining permission.
 */

package nodenet;

import java.awt.*;
import java.util.Vector;

/**
 * This class is the NodeBehavior for a TerminatorNode.<p>
 *
 * @author Todd C. Parnell, tparnell@ai.mit.edu
 * @version $Id: TerminatorNodeBehavior.java,v 1.5 2004/01/14 21:43:17 gus Exp $
 */
class TerminatorNodeBehavior implements NodeBehavior, Counter {
    public static final String BEHAVIOR_NAME = "Terminator";
    
    private int num_des;
    
    public void transmitPacket(InputChannelVector inputChannels, OutputChannelVector outputChannels ) {
        int numInputChannels = inputChannels.size();
        int numOutputChannels = outputChannels.size();
        
        int in_number;
        boolean found = false;
        Object o;
        
        // If no inputs
        if (numInputChannels == 0) return;
        
        // sometimes, just bail
        if (Math.random() < 0.6) return;
        
        for (int i = 0; i < 20; i++) {
            in_number = (int)(Math.random() * numInputChannels);
            try {
                o = inputChannels.elementAt(in_number).readObject(); // ()?
                this.num_des += 1;
            } catch(ChannelEmptyException exc) {
                return;
            } catch(ChannelDisabledException exc2) {
                return;
            }
        }
    }
    
    public int getCount() {
        return(this.num_des);
    }
    
    public void resetCount() {
        this.num_des = 0;
    }
}

/*
 * $Log: TerminatorNodeBehavior.java,v $
 * Revision 1.5  2004/01/14 21:43:17  gus
 * more javadoc, plus reformat
 *
 * Revision 1.4  2004/01/14 20:23:21  gus
 * Javadoc and comment cleanup
 *
 * Revision 1.3  2004/01/13 19:35:27  gus
 * Simulation Panel, channel and node are all Resettable now.
 * CountingNodeBehavior refactored into Counter
 *
 * Revision 1.2  2004/01/07 18:32:47  gus
 * Ripped out ControlPanel and NodeNetRadioButton.
 * Replaced them with NodeTypeSelector, which provides
 * The loaded User NodeBehavior types as an indexed
 * property. Simulation panel now implements PropertyChangeListener
 * to listen for changes in the selected node behavior. The
 * buttons in the display are now JToggleButtons, and will
 * use the public field BEHAVIOR_NAME to lable the buttons
 * if it is declared for any NodeBehavior.
 *
 * Revision 1.1  2002/06/13 17:33:21  gus
 * Moved all java files into the nodenet directory (who let them out anyway?)
 *
 * Revision 1.1.1.1  2002/06/05 21:56:35  root
 * CS101 comes to Olin finally.
 *
 * Revision 1.4  2000/05/09 06:03:54  mharder
 * Changed packagename from nodeNet to nodenet.
 *
 * Revision 1.3  1999/08/04 09:08:55  jsmthng
 * Added javadoc comments to InputChannelVector and OutputChannelVector;
 * finished updating the rest of the nodeNet package to reflect new
 * changes in name and code.
 *
 * Modified index.html to reflect the new nodeNet code, as well as to
 * clarify some parts of the problem set.
 *
 * Revision 1.1  1999/07/30 01:09:24  jsmthng
 * Renaming BinSort package to nodeNet; moving directories and files as
 * necessary.
 *
 * Revision 1.4  1998/08/12 19:29:44  tparnell
 * Another pass after comments from las & natashao.  Added support to
 * dynamically add NodeBehaviors.  Add keyboard shortcuts to menus.  Added
 * workaround to jdk bug relating to lighweight components.  Misc other
 * bugfixes.
 *
 * Revision 1.3  1998/08/10 17:45:50  tparnell
 * Revision to use JDK1.2 and Swing.  Redesign of GUI.  Removed old kludge
 * for file I/O and replaced with object serialization.  Channel no longer
 * requires animacy.  Removed unnessary dependencies between classes.
 * Added ability to configure channel's latency and capacity.  Added
 * javadoc to all files.  General code cleanup.
 *
 */
