/*
 * DestinationValidator.java
 *
 * Developed for the "Rethinking CS101" project. See http://www.cs101.org, the
 * CS101 homepage or email las@olin.edu.
 *
 * Created on February 24, 2003, 9:08 AM
 */

package nodenet.registrar;

import nodenet.NodeConfigurationBean;

import nodenet.registrar.NodeNameRegistry;

import java.io.Serializable;
import java.io.ObjectStreamException;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;

/**
 * A quick implementation of a class that validates destination values
 * for <code>NodeConfigurationBeans</code>. It is a singleton class of
 * which there should never be more than one instance.
 *
 * FIXME: This should be broken out to a strategy pattern similar to
 * the <code>NodeNameRegistry</code> class. This class is not in current use
 * but may be used in future versions.
 *
 * @author  Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: DestinationValidator.java,v 1.6 2004/01/14 20:23:21 gus Exp $
 */
public class DestinationValidator implements VetoableChangeListener, Serializable {
    
    private static final DestinationValidator SINGLETON = new DestinationValidator();
    
    /** Creates a new instance of DestinationValidator */
    private DestinationValidator() {
    }
    
    /**
     * Get the one, the only, the DestinationValidator.
     *
     * @return The exact same DestinationValidator object on every call within
     *         a given instance of the VM.
     */
    
    public static DestinationValidator getInstance() {
        return SINGLETON;
    }
    
    /**
     * Decide whether or not to veto a change to the dest property of a
     * <code>NodeConfigurationBean</code>.
     */
    public void vetoableChange(PropertyChangeEvent pce) throws PropertyVetoException {
        if (pce.getPropertyName().equals(NodeConfigurationBean.PROP_DEST)) {
            if (!NodeNameRegistry.getInstance().isRegistered((String)pce.getNewValue()) &&
            !pce.getNewValue().equals("")) {
                
                throw new PropertyVetoException("Destination Node '" +
                pce.getNewValue()+"' doesn't exist!",pce);
            }
        }
        
    }
    
    // This allows us to deserialize objects containing a reference to the
    // NodeNameRegistry and never cause duplicate registries. This technique
    // means that when nodes are deserialized they need to re-register
    // with the existing registry.
    private Object readResolve() throws ObjectStreamException {
        return SINGLETON;
    }
    
}

/*
 * $Log: DestinationValidator.java,v $
 * Revision 1.6  2004/01/14 20:23:21  gus
 * Javadoc and comment cleanup
 *
 * Revision 1.5  2003/02/25 22:33:28  gus
 * javadoc enhancements
 *  Modified Files:
 *  	nodenet/NodeConfigurationBean.form
 *  	nodenet/NodeConfigurationBean.java
 *  	nodenet/registrar/DestinationValidator.java
 *  	nodenet/registrar/UniqueNodeNamePolicy.java
 *
 * Revision 1.4  2003/02/25 22:17:30  gus
 * javadocs
 *
 * Revision 1.3  2003/02/25 16:02:49  gus
 * Make this class serializeable so we can save things again
 *
 * Revision 1.2  2003/02/24 16:27:47  gus
 * added Log comment
 *
 */